@@ -4,6 +4,8 @@ from typing import Dict, Tuple, List, Union
 
 from mangadex import (Manga, Tag, Chapter, User, UserError, ChapterError, Author, ScanlationGroup, CoverArt, CustomList, URLRequest, TagError)
 
+import re #for validating email and prevent spam
+
 class Api():
     def __init__(self, timeout = 5):
         self.URL = 'https://api.mangadex.org'
@@ -18,7 +20,7 @@ class Api():
         self.bearer = bearer
 
     @staticmethod
-    def _parse_manga_params(params : dict):
+    def _parse_manga_params(params : dict) -> dict:
         if "authors" in params:
             temp = params.pop("authors")
             params["authors[]"] = temp
@@ -55,6 +57,10 @@ class Api():
         if "translatedLanguage" in params:
             temp = params.pop("translatedLanguage")
             params["translatedLanguage[]"] = temp
+        if "status" in params:
+            params["status[]"] = params.pop("status")
+        if "contentRating" in params:
+            params["contentRating[]"] = params.pop("contentRating")
         return params
 
     def get_manga_list(self, **kwargs) -> List[Manga]:
@@ -167,26 +173,30 @@ class Api():
         ------------
         `Manga`. A manga object if `ObjReturn` is set to `True`
         """
-        kwargs = self._parse_manga_params(kwargs)
+        params = self._parse_manga_params(kwargs)
         url = f"{self.URL}/manga"
-        kwargs["title"] = title
-        resp = URLRequest._request_url(url, "POST", params=kwargs, headers=self.bearer, timeout = self.timeout)
+        params["title"] = title
+        resp = URLRequest._request_url(url, "POST", params=params, headers=self.bearer, timeout = self.timeout)
         return  Manga._create_manga(resp)
 
-    def get_manga_volumes_and_chapters(self, id : str) -> Dict[str, str]:
+    def get_manga_volumes_and_chapters(self, id : str, **kwargs) -> Dict[str, str]:
         """
         Get a manga volumes and chapters
 
         Parameters
         ------------
         id : `str`. The manga id
+        translatedLanguage : `List[str]`
 
         Returns
         ------------
         `Dict[str, str]`. A dictionary with the volumes and the chapter id's
         """
+        params = None
+        if "translatedLanguage" in kwargs:
+            params = {"translatedLanguage[]" : kwargs["translatedLanguage"]}
         url = f"{self.URL}/manga/{id}/aggregate"
-        resp = URLRequest._request_url(url, "GET", timeout = self.timeout)
+        resp = URLRequest._request_url(url, "GET", timeout = self.timeout, params = params)
         return resp["volumes"]
 
     def update_manga(self, id : str, ObjReturn : bool = False ,**kwargs) -> Manga:
@@ -234,9 +244,6 @@ class Api():
         ------------
         id : `str`. The manga id
 
-        Returns
-        -----------
-        `None`
         """
         url = f"{self.URL}/manga{id}"
         URLRequest._request_url(url, "DELETE", headers=self.bearer, timeout= self.timeout)
@@ -307,6 +314,17 @@ class Api():
         resp = URLRequest._request_url(url, "GET", timeout = self.timeout, params = kwargs)
         return Chapter._create_chapter_list(resp)
 
+    @staticmethod
+    def _parse_chapter_list_args(params : Dict[str, str]) -> Dict[str,str]:
+        if "groups" in params:
+            params["groups[]"] =  params.pop("groups")
+        if "volume" in params:
+            params["volume[]"] = params.pop("volume")
+        if "translatedLanguage" in params:
+            params["translatedLanguage[]"] = params.pop("translatedLanguage")
+
+        return params
+
     def chapter_list(self, **kwargs) -> List[Chapter]:
         """
         The list of chapters. To get the chpaters of a specific manga the manga parameter must be provided
@@ -318,12 +336,12 @@ class Api():
         limit : `int`
         offset : `int`
         title : `str`
-        groups : `[str]`
+        groups : `List[str]`
         uploader : `str`    
         manga : `str`
-        volume : `str`
+        volume : `str | List[str]`
         chapter : `str`
-        translatedLanguage : `str`
+        translatedLanguage : `List[str]`
         createdAtSince : `str`. Datetime String with the following format YYYY-MM-DDTHH:MM:SS
         updatedAtSince : `str`. Datetime String with the following format YYYY-MM-DDTHH:MM:SS
         publishAtSince : `str`. Datetime String with the following format YYYY-MM-DDTHH:MM:SS
@@ -337,11 +355,9 @@ class Api():
         `ApiError` \n
         `ChapterError`
         """
-        if "groups" in kwargs:
-            temp = kwargs.pop("groups")
-            kwargs["groups[]"] = temp
+        params =  Api._parse_chapter_list_args(kwargs)
         url = f"{self.URL}/chapter"
-        resp = URLRequest._request_url(url, "GET", timeout= self.timeout, params= kwargs)
+        resp = URLRequest._request_url(url, "GET", timeout= self.timeout, params= params)
         return Chapter._create_chapter_list(resp)
 
     def get_chapter(self, id: str) -> Chapter:
@@ -374,7 +390,7 @@ class Api():
         ------------
         limit : `int`
         offset : `int`
-        ids : `string`. Array of ids 
+        ids : List[`str`]. Array of ids 
         name : `str`
 
         Returns
@@ -386,6 +402,9 @@ class Api():
         `ApiError` \n
         `AuthorError`
         """
+        if "ids" in kwargs:
+            kwargs["ids[]"] = kwargs.pop("ids")
+        
         url = f"{self.URL}/author"
         resp = URLRequest._request_url(url, "GET", timeout=self.timeout ,params = kwargs)
         return Author._create_authors_list(resp)
@@ -462,10 +481,6 @@ class Api():
         Parameters
         ---------------
         id : `str`. Required. The author id
-
-        Returns
-        --------------
-        `None`
         """
         url = f"{self.URL}/author/{id}"
         URLRequest._request_url(url, "DELETE", headers=self.bearer)
@@ -577,10 +592,13 @@ class Api():
 
         offset `int`
 
+        translatedLanguage : `List[str]`
         Returns
         -------------
         `List[ScanlationGroup]`
         """
+        if "translatedLanguage" in kwargs: 
+            kwargs['translatedLanguage[]'] = kwargs.pop("translatedLanguage")
         url = f"{self.URL}/user/follows/group"
         resp = URLRequest._request_url(url, "GET", timeout = self.timeout,  params=kwargs, headers= self.bearer)
         return ScanlationGroup._create_group_list(resp)
@@ -598,7 +616,6 @@ class Api():
         Returns
         -------------
         `List[User]`
-        
         """
         url = f"{self.URL}/user/follows/user"
         resp = URLRequest._request_url(url, "GET", timeout= self.timeout ,params=kwargs, headers=self.bearer)
@@ -615,7 +632,6 @@ class Api():
         Returns
         ------------
         `str` The manga reading status
-
         """
         url = f"{self.URL}/manga/{id}/status"
         resp = URLRequest._request_url(url, "GET", headers=self.bearer, timeout=self.timeout)
@@ -627,12 +643,10 @@ class Api():
 
         Parameters
         ------------
-
         status : `str`. Optional. Values : `"reading"` `"on_hold"` `"plan_to_read"` `"dropped"` `"re_reading"` `"completed"`
 
         Returns
         -----------
-
         `Dict[str,str]` A dictionary with the Manga id and its status
         """
         url = f"{self.URL}/manga/status"
@@ -643,15 +657,11 @@ class Api():
         """
         Follow a manga
 
-        Paramerets
+        Parameters
         --------------
 
         id : `str`. The manga id
 
-        Returns
-        -------------
-        `None`
-
         Raises
         -------------
         `ApiError`
@@ -667,10 +677,6 @@ class Api():
         -------------
         id : `str`. The manga id
 
-        Returns
-        ------------
-        `None`
-
         Raises
         -----------
         `ApiError`
@@ -688,10 +694,6 @@ class Api():
         id : `str`. The manga id
         status : `str`. Values : `"reading"` `"on_hold"` `"plan_to_read"` `"dropped"` `"re_reading"` `"completed"`
 
-        Returns
-        -------------
-        `None`
-
         Raises
         -------------
         `ApiError`
@@ -707,10 +709,6 @@ class Api():
         --------------
         id : `str`. The manga id
         listId : `str`. The list id
-
-        Returns
-        -------------
-        `None`
         """
         url = f"{self.URL}/{id}/list{listId}"
         URLRequest._request_url(url, "POST", headers=self.bearer, timeout=self.timeout)
@@ -723,10 +721,6 @@ class Api():
         ------------
         id : `str`. The manga id
         listId : `str`. The list id
-
-        Returns
-        ------------
-        `None`
         """
         url = f"{self.URL}/manga/{id}/list/{listId}"
         URLRequest._request_url(url, "DELETE", headers=self.bearer, timeout=self.timeout)
@@ -742,10 +736,6 @@ class Api():
         name : `str`. The custom list name
         visibility : `str. The visibility of the custom list
         manga : `List[str]`. List of manga ids
-
-        Returns
-        -----------
-        `None`
         """
         url = f"{self.URL}/list"
         params = {"name" : name, "version" : version}
@@ -753,18 +743,22 @@ class Api():
         params["manga[]"] = manga
         URLRequest._request_url(url, "POST", params=params, timeout=self.timeout)
 
-    def get_customlist(self, id : str) -> CustomList:
+    def get_customlist(self, id : str, **kwargs) -> CustomList:
         """
         Get a custom list by its id
 
         Parameters
         ------------
         id : `str`. The id of the custom list
-
+        limit : int
+        offset : int
+        translatedLanguage : List[str]
+        createdAtSince : `str`. Datetime String with the following format YYYY-MM-DDTHH:MM:SS
+        updatedAtSince : `str`. Datetime String with the following format YYYY-MM-DDTHH:MM:SS
+        publishAtSince : `str`. Datetime String with the following format YYYY-MM-DDTHH:MM:SS
         Returns
         ------------
         `CustomList`
-        
         """
         url = f"{self.URL}/list/{id}"
         resp = URLRequest._request_url(url, "GET", headers=self.bearer, timeout=self.timeout)
@@ -798,10 +792,6 @@ class Api():
         Parameters
         ------------
         id : `str`. The custom list id
-
-        Returns
-        ----------
-        `None`
         """
         url = f"{self.URL}/list{id}"
         URLRequest._request_url(url, "DELETE", headers=self.bearer, timeout=self.timeout)
@@ -820,7 +810,6 @@ class Api():
         Returns
         ----------
         `List[CustomList]`
-
         """
         url = f"{self.URL}/user/list"
         resp = URLRequest._request_url(url, "GET", params=kwargs, headers=self.bearer, timeout=self.timeout)
@@ -851,7 +840,7 @@ class Api():
         """
         Get the chapter feed of a given custom list. 
 
-        Paramters
+        Parameters
         ------------
         id : `str`. The custom list id
 
@@ -872,25 +861,71 @@ class Api():
         resp = URLRequest._request_url(url, "GET", params=kwargs, headers=self.bearer, timeout = self.timeout)
         return  Chapter._create_chapter_list(resp)
 
+
+    @staticmethod
+    def _parse_coverart_params(params : Dict[str,str]) -> Dict[str, str]:
+        if "manga" in params:
+            params["manga[]"] = params.pop("manga")
+        if "ids" in params:
+            params["ids[]"] = params.pop("ids")
+        if "uploaders" in params:
+            params["uploaders[]"] = params.pop("uploaders")
+
+        return params
     def get_coverart_list(self, **kwargs):
         """
         Get the list of cover arts (like the manga feed)
+
+        Optional parameters
+        -------------------------
+
+        manga : List[str]. Manga ids
+        
+        ids : List[str]. Cover ids
+
+        uploaders : List[str]. User ids
+
         """
-        if "manga" in kwargs:
-            temp = kwargs.pop("manga")
-            kwargs["manga[]"] = temp 
+
+        params = Api._parse_coverart_params(kwargs)
         url = f"{self.URL}/cover"
-        resp = URLRequest._request_url(url, "GET", params=kwargs, timeout= self.timeout)
+        resp = URLRequest._request_url(url, "GET", params = params, timeout= self.timeout)
         return CoverArt._createCoverImageList(resp)
     
     def get_cover(self, coverId : str) -> CoverArt:
-        
+        """
+        Gets a cover image
+
+        Parameters
+        --------------
+        coverId : `str`. The cover id
+
+        Returns
+        --------------
+        `CoverArt`. A cover art object
+        """
         url = f"{self.URL}/cover/{coverId}"
         resp = URLRequest._request_url(url, "GET", timeout=self.timeout)
         return CoverArt._createCoverImage(resp)
         
-    def upload_cover(self, manga_id : str, file : str, ObjReturn : bool = False):
+    def upload_cover(self, manga_id : str, filename : str, ObjReturn : bool = False) -> Union[CoverArt, None]:
+        """
+        Uploads a cover 
+
+        Parameters
+        --------------
+        manga_id : `str`
+        filename : `str`
+        ObjReturn : `bool`
+
+        Returns
+        -------------
+        `CoverArt` if `ObjReturn` set to `True`
+        """
         url = f"{self.URL}/cover/{manga_id}"
+        with open(filename, 'rb') as f:
+            file = f.read()
+
         resp = URLRequest._request_url(url, "POST", params = {"file" : file}, headers= self.bearer, timeout=self.timeout)
         return CoverArt._createCoverImage(resp) if ObjReturn else None
 
@@ -918,7 +953,7 @@ class Api():
             params["description"] = description
 
         url = f"{self.URL}/cover/{coverId}"
-        resp = URLRequest._request_url(url, "PUt", params=params, headers=self.bearer, timeout=self.timeout)
+        resp = URLRequest._request_url(url, "PUT", params=params, headers=self.bearer, timeout=self.timeout)
         return CoverArt._createCoverImage(resp) if ObjReturn else None
     
     def delete_cover(self, coverId : Union[str , CoverArt]):
@@ -934,4 +969,86 @@ class Api():
         if type(coverId) == CoverArt:
             coverId = coverId.id
         url = f"{self.URL}/cover/{coverId}"
-        URLRequest._request_url(url, "DELETE", headers= self.bearer, timeout=self.timeout)
\ No newline at end of file
+        URLRequest._request_url(url, "DELETE", headers= self.bearer, timeout=self.timeout)
+
+    def create_account(self, username : str, password : str, email : str, ObjReturn : bool = False) -> Union[User, None]:
+        """
+        Creates an account
+
+        Parameters
+        ---------------
+        username : `str`.
+        password : `str`.
+        email : `str`.
+        ObjReturn : `bool`.
+
+        Return
+        -------------
+        `User` if `ObjReturn` set to `True`
+        """
+        url = f"{self.URL}/account/create"
+        email_regex = '^(\w|\.|\_|\-)+[@](\w|\_|\-|\.)+[.]\w{2,3}$' # regular expression for email
+        if re.search(email_regex, email) is None:
+            raise ValueError("The email provided is not valid")
+        
+        if len(password) < 8:
+            raise ValueError("Password must have at least 8 characters")
+        params = {"username" : username, "password" : password, "email" : email}
+        resp = URLRequest._request_url(url, "POST", timeout=self.timeout, params=params)
+        return User._create_user(resp["data"]) if ObjReturn else None
+    
+    def activate_account(self, code : str):
+        """
+        Handles the activation code for the account creation
+
+        Parameters
+        ------------
+        code : `str`. The activation code sent to the email provided
+        """
+        url = f"{self.URL}/account/activate/{code}"
+        URLRequest._request_url(url, "GET", timeout=self.timeout)
+    
+    def resend_activation_code(self, email : str):
+        """
+        Resends the activation code to another email
+
+        Parameters
+        -----------------
+        email : `str`.
+        """
+        email_regex = '^(\w|\.|\_|\-)+[@](\w|\_|\-|\.)+[.]\w{2,3}$' # regular expression for email
+        if re.search(email_regex, email) is not None:
+            raise ValueError("The email provided is not valid")
+        params = {"email" : email}
+        url = f"{self.URL}/account/activate/resend"
+        URLRequest._request_url(url, "POST", timeout=self.timeout, params=params)
+    
+    def recover_account(self, email : str):
+        """
+        Recover an existing account
+
+        Parameters
+        --------------
+        email : `str`.
+        """
+        email_regex = '^(\w|\.|\_|\-)+[@](\w|\_|\-|\.)+[.]\w{2,3}$'
+        if re.search(email_regex, email) is None:
+            raise ValueError("The email provided is not valid")
+        params = {"email" : email}
+        url = f"{self.URL}/account/recover"
+        URLRequest._request_url(url, "POST", params=params)
+
+    def complete_account_recover(self, code : str, newPassword : str):
+        """
+        Completes the account recover process
+
+        Parameters
+        --------------
+        code : `str`. The code sended to the email given in `recover_account`
+        newPassword : `str`. The new password for the account
+        """
+        if len(newPassword) < 8:
+            raise ValueError("Password must have at least 8 characters")
+        url = f"{self.URL}/account/recover/{code}"
+        params = {"newPassword" : newPassword}
+        URLRequest._request_url(url, "POST", timeout=self.timeout, params=params)
\ No newline at end of file
