@@ -27,21 +27,25 @@ class Manga():
         self.version = 1
         self.createdAt : datetime = ""
         self.updatedAt : datetime = ""
-        self.author : str = ""
-        self.artist : str = ""
+        self.authorId : List[str] = []
+        self.artistId : List[str] = []
         self.coverId : str = ""
 
     def _MangaFromDict(self, data : dict):
-        if data["data"]["type"] != 'manga' or not data:
-            raise MangaError("The data provides is not a Manga")
+        if data["type"] != 'manga' or not data:
+            raise MangaError(data=data, message="The data probvided is not a Manga")
         
-        attributes = data["data"]["attributes"]
+        attributes = data["attributes"]
 
-        self.id = data["data"]["id"]
+        self.id = data["id"]
         self.title = attributes["title"]
         self.altTitles = attributes["altTitles"]
         self.description = attributes["description"]
-        self.isLocked = attributes["isLocked"]
+        try:
+            self.isLocked = attributes["isLocked"]
+        except KeyError:
+            pass
+        
         self.links = attributes["links"]
         self.originalLanguage = attributes["originalLanguage"]
         self.lastVolume = attributes["lastVolume"]
@@ -50,47 +54,70 @@ class Manga():
         self.status = attributes["status"]
         self.year = attributes["year"]
         self.contentRating = attributes["contentRating"]
-        self.tags = attributes["tags"]
+        self.tags = Tag._create_tag_list(attributes["tags"])
         self.createdAt = parse(attributes["createdAt"])
         self.updatedAt = parse(attributes["updatedAt"])
 
-        self.authorId  = data["relationships"][0]["id"]
-        self.artistId  = data["relationships"][1]["id"]
-        self.coverId = data["relationships"][2]["id"]
+        for elem in data["relationships"]:
+            if elem['type'] == 'author':
+                self.authorId.append(elem['id'])
+            elif elem['type'] == 'artist':
+                self.artistId.append(elem['id'])
+            elif elem['type'] == 'cover_art':
+                self.coverId = elem['id']
+
 
     @staticmethod
     def _create_manga(elem) -> 'Manga':
         manga = Manga()
         manga._MangaFromDict(elem)
         return manga
-        
+    
     @staticmethod
     def _create_manga_list(resp) -> List['Manga']:
-        resp = resp["results"]
+        resp = resp["data"]
         manga_list = []
         for elem in resp:
             manga_list.append(Manga._create_manga(elem))
         return manga_list
 
+    def __eq__(self, other : 'Manga') -> bool:
+        my_vals = [self.id, self.title, self.createdAt, self.authorId]
+        other_vals = [other.id, other.title, other.createdAt, other.authorId]
+        return all((me == other for me, other in zip(my_vals, other_vals)))
+    
+    def __ne__(self, other: object) -> bool:
+        return not self.__eq__(other)
+
     def __repr__(self) -> str:
         temp1 = f"Manga(id = {self.id}, title = {self.title}, altTitles = {self.altTitles}, description = {self.description}, isLocked = {self.isLocked}, links = {self.links}, originalLanguage = {self.originalLanguage} \n"
         temp2 = f"lastVolume = {self.lastVolume}, lastChapter = {self.lastChapter}, publicationDemographic = {self.publicationDemographic}, status = {self.status}, year = {self.year}, contentRating = {self.contentRating} \n"
-        temp3 = f"createdAt = {self.createdAt}, uploadedAt = {self.updatedAt}), authorId = {self.author}, artistId = {self.artist}, coverId = {self.coverId}"
+        temp3 = f"createdAt = {self.createdAt}, uploadedAt = {self.updatedAt}), authorId = {self.authorId}, artistId = {self.artistId}, coverId = {self.coverId}"
         return temp1 + temp2 + temp3
 
 class Tag():
     def __init__(self) -> None:
         self.id : str= ""
         self.name : Dict[str, str] = {}
+        self.description : str = ""
+        self.group : str = ""
 
     def _TagFromDict(self, data : dict):
-        if data["data"]["type"] != 'tag' or not data:
-            raise TagError("The data provided is not a Tag")
         
-        attributes = data["data"]["attributes"]
+        try:
+            data = data["data"]
+        except KeyError:
+            pass
 
-        self.id = data["data"]["id"]
+        if data["type"] != 'tag' or not data:
+            raise TagError(data=data, message="The data provided is not a Tag")
+        
+        attributes = data["attributes"]
+
+        self.id = data["id"]
         self.name = attributes["name"]
+        self.description = attributes["description"]
+        self.group = attributes["group"]
 
     @staticmethod
     def _create_tag(elem) -> 'Tag':
@@ -101,10 +128,18 @@ class Tag():
     @staticmethod
     def _create_tag_list(resp) -> List['Tag']:
         tag_list = []
-        for tag in resp:
+        for tag in resp["data"]:
             tag_list.append(Tag._create_tag(tag))
         return tag_list
 
+    def __eq__(self, other : 'Tag') -> bool:
+        my_vals = [self.id, self.name]
+        other_vals = [other.id, other.name]
+        return all((me == other for me,other in zip(my_vals, other_vals)))
+
+    def __ne__(self, other: 'Tag') -> bool:
+        return not self.__eq__(other)
+
     def __repr__(self) -> str:
         return f"Tag(id = {self.id}, name = {self.name})"
 
@@ -115,7 +150,7 @@ class Chapter():
         self.volume : str = ""
         self.chapter : float = None
         self.Mangaid : str = ""
-        self.sacanlation_group_id : str = ""
+        self.scanlation_group_id : str = ""
         self.translatedLanguage : str= ""
         self.hash : str = ""
         self.data : List[str] = []
@@ -125,22 +160,28 @@ class Chapter():
         self.publishAt : datetime = ""
     
     def _ChapterFromDict(self, data):
-        if data["data"]["type"] != 'chapter' or not data:
-            raise ChapterError("The data probvided is not a Tag")
 
-        attributes = data["data"]["attributes"]
+        try:
+            data = data["data"]
+        except KeyError:
+            pass
 
-        self.id = data["data"]["id"]
+        if data["type"] != 'chapter' or not data:
+            raise ChapterError(data = data, mmessage="The data provided is not a Chapter")
+
+        attributes = data["attributes"]
+
+        self.id = data["id"]
         self.title = attributes["title"]
         self.volume = attributes["volume"]
-        self.chapter = float(attributes["chapter"])
+        self.chapter = float(attributes["chapter"]) if attributes['chapter'] is not None else None
         self.translatedLanguage = attributes["translatedLanguage"]
         self.hash = attributes["hash"]
         self.data = attributes["data"]
         self.publishAt = parse(attributes["publishAt"])
         self.createdAt = parse(attributes["createdAt"])
         self.updatedAt = parse(attributes["updatedAt"])
-        self.sacanlation_group_id = data["relationships"][0]["id"]
+        self.scanlation_group_id = data["relationships"][0]["id"]
         self.Mangaid = data["relationships"][1]["id"]
         self.uploader = data["relationships"][2]["id"]
 
@@ -176,15 +217,23 @@ class Chapter():
     
     @staticmethod
     def _create_chapter_list(resp) -> List['Chapter']:
-        resp = resp["results"]
+        resp = resp["data"]
         chap_list = []
         for elem in resp:
             chap_list.append(Chapter._create_chapter(elem))
         return chap_list
 
+    def __eq__(self, other: 'Chapter') -> bool:
+        my_vals = [self.id, self.hash, self.Mangaid, self.chapter]
+        other_vals = [other.id, other.hash, other.Mangaid, other.chapter]
+        return all((me == other for me,other in zip(my_vals, other_vals)))
+
+    def __ne__(self, other: 'Chapter') -> bool:
+        return not self.__eq__(other)
+    
     def __repr__(self) -> str:
         temp1 =  f"Chapter(id = {self.id}, title = {self.title}, volume = {self.volume}, chapter = {self.chapter}, translatedLanguage = {self.translatedLanguage}, hash = {self.hash} \n"
-        temp2 = f"data = List[filenames], publishAt = {self.publishAt}, createdAt = {self.createdAt}, uploadedAt = {self.updatedAt}, sacanlation_group_id = {self.sacanlation_group_id}, Mangaid = {self.Mangaid}, uploader = {self.uploader})"
+        temp2 = f"data = List[filenames], publishAt = {self.publishAt}, createdAt = {self.createdAt}, uploadedAt = {self.updatedAt}, scanlation_group_id = {self.scanlation_group_id}, Mangaid = {self.Mangaid}, uploader = {self.uploader})"
         return temp1 + temp2
 
 class User():
@@ -193,12 +242,15 @@ class User():
         self.username :str = ""
 
     def _UserFromDict(self, data):
-        if data["data"]["type"] != "user" or not data:
-            raise UserError("The data provided is not an author")
-        
-        attributes = data["data"]["attributes"]
+        if "data" in data:
+            data = data["data"]
 
-        self.id = data["data"]["id"]
+        if data["type"] != "user" or not data:
+            raise UserError(data = data, message="The data provided is not a User")
+        
+        attributes = data["attributes"]
+
+        self.id = data["id"]
         self.username = attributes["username"]
 
     @staticmethod
@@ -209,12 +261,20 @@ class User():
 
     @staticmethod    
     def _create_user_list(resp) -> List['User']:
-        resp = resp["results"]
+        resp = resp["data"]
         user_list = []
         for elem in resp:
             user_list.append(User._create_user(elem))
         return user_list
 
+    def __eq__(self, other: 'User') -> bool:
+        my_vals = [self.id, self.username]
+        other_vals = [other.id, other.username]
+        return all((me == other for me,other in zip(my_vals, other_vals)))
+
+    def __ne__(self, other: 'User') -> bool:
+        return not self.__eq__(other)
+
     def __repr__(self) -> str:
         return f"User(id = {self.id}, username = {self.username})"
 
@@ -226,19 +286,27 @@ class Author():
         self.bio : Dict[str,  str] = {}
         self.createdAt : datetime = ""
         self.updatedAt : datetime = ""
+        self.mangas : List[str] = []
     
     def _AuthorFromDict(self, data):
-        if data["data"]["type"] != "author" or not data:
-            raise AuthorError("The data provided is not an author")
-    
-        attributes = data["data"]["attributes"]
 
-        self.id = data["data"]["id"]
+        try:
+            data = data["data"]
+        except KeyError:
+            pass
+
+        if data["type"] != "author" or not data:
+            raise AuthorError(data = data, message= f"The data provided is not Author is : {data['type']}")
+    
+        attributes = data["attributes"]
+
+        self.id = data["id"]
         self.name = attributes["name"]
         self.imageUrl = attributes["imageUrl"]
         self.bio = attributes["biography"]
         self.createdAt = parse(attributes["createdAt"])
         self.updatedAt = parse(attributes["updatedAt"])
+        self.mangas =  [manga["id"] for manga in data["relationships"] if manga["type"] == "manga"] # better keep it like this to not consume computing time
 
     @staticmethod
     def _create_author(elem) -> 'Author':
@@ -248,12 +316,20 @@ class Author():
 
     @staticmethod
     def _create_authors_list(self, resp) -> List['Author']:
-        resp = resp["results"]
+        resp = resp["data"]
         authors_list = []
         for elem in resp:
             authors_list.append(Author._create_author(elem))
         return authors_list
 
+    def __eq__(self, other: 'Author') -> bool:
+        my_vals = [self.id ,self.name]
+        other_vals = [other.id, other.name]
+        return all((me == other for me,other in zip(my_vals, other_vals)))
+
+    def __ne__(self, other: 'Author') -> bool:
+        return not self.__eq__(other)
+
     def __repr__(self) -> str:
         return f"Author(id = {self.id}, name = {self.name}, imageUrl = {self.imageUrl}, createdAt = {self.createdAt}, updatedAt = {self.updatedAt})"
 
@@ -267,17 +343,24 @@ class ScanlationGroup():
 
     def _ScanlationFromDict(self, data):
         
-        if data["data"]["type"] != "scanlation_group" or not data:
+        if data["type"] != "scanlation_group" or not data:
             raise ScanlationGroupError("The data provided is not an scanlation group")
 
-        attributes = data["data"]["attributes"]
-
-        self.id = data["data"]["id"]
+        attributes = data["attributes"]
+        relationships = data["relationships"]
+        self.id = data["id"]
         self.name = attributes["name"]
 
         leader = User()
-        leader.id = attributes["leader"]["id"]
-        leader.username = attributes["leader"]["attributes"]["username"] #didn't use the _UserFromDict method becasue the api response is different
+        for elem in relationships:
+            try:
+                if elem["type"] == "leader":
+                    leader.id = elem["id"]
+                    break
+            except KeyError:
+                continue
+
+        # leader.username = attributes["leader"]["attributes"]["username"] #didn't use the _UserFromDict method becasue the api response is different
         self.leader =  leader
 
         self.createdAt = parse(attributes["createdAt"])
@@ -290,13 +373,21 @@ class ScanlationGroup():
         return group
 
     @staticmethod
-    def _create_group_list(resp)-> List['ScanlationGroup']:
-        resp = resp["results"]
+    def _create_group_list(resp) -> List['ScanlationGroup']:
+        resp = resp["data"]
         group_list = []
         for elem in resp:
             group_list.append(ScanlationGroup._create_group(elem))
         return group_list  
 
+    def __eq__(self, other: 'ScanlationGroup') -> bool:
+        my_vals = []
+        other_vals = []
+        return all((me == other for me,other in zip(my_vals, other_vals)))
+    
+    def __ne__(self, other: 'ScanlationGroup') -> bool:
+        return not self.__eq__(other)
+        
     def __repr__(self) -> str:
         return f"ScanlationGroup(id = {self.id}, name = {self.name}, leader = {self.leader}, createdAt = {self.createdAt}, updatedAt = {self.updatedAt})"
 
@@ -309,12 +400,12 @@ class CustomList():
         self.mangas : List[Manga] = None
 
     def _ListFromDict(self, data):
-        if data["data"]["type"] != "custom_list" or not data:
+        if data["type"] != "custom_list" or not data:
             raise CustomListError("The data provided is not a Custom List")
         
-        attributes = data["data"]["attributes"]
+        attributes = data["attributes"]
 
-        self.id = data["data"]["id"]
+        self.id = data["id"]
         self.name = attributes["name"]
         self.visibility = attributes["visibility"]
         self.owner = User()
@@ -328,7 +419,7 @@ class CustomList():
 
     @staticmethod
     def _create_customlist_list(resp) -> List['CustomList']:
-        resp = resp["results"]
+        resp = resp["data"]
         custom_lists = []
         for elem in resp:
             custom_lists.append(CustomList._create_customlist(elem))
@@ -348,12 +439,12 @@ class CoverArt():
         self.mangaId : str = None
 
     def _CoverFromDict(self, data):
-        if data["data"]["type"] != "cover_art" or not data:
+        if data["type"] != "cover_art" or not data:
             raise CoverArtError("The data provided is not a Custom List")
         
-        attributes = data["data"]["attributes"]
+        attributes = data["attributes"]
 
-        self.id = data["data"]["id"]
+        self.id = data["id"]
         self.volume = attributes["volume"]
         self.fileName = attributes["fileName"]
         self.description = attributes["description"]
@@ -390,7 +481,7 @@ class CoverArt():
 
     @staticmethod
     def _createCoverImageList(resp) -> List['CoverArt']:
-        resp = resp["results"]
+        resp = resp["data"]
         coverimage_list = []
         for elem in resp:
             coverimage_list.append(CoverArt._createCoverImage(elem))
